@@ -25,11 +25,11 @@ int pthread_setcancelstate(int state, int * oldstate)
   pthread_descr self = thread_self();
   if (state < PTHREAD_CANCEL_ENABLE || state > PTHREAD_CANCEL_DISABLE)
     return EINVAL;
-  if (oldstate != NULL) *oldstate = self->p_cancelstate;
-  self->p_cancelstate = state;
-  if (self->p_canceled &&
-      self->p_cancelstate == PTHREAD_CANCEL_ENABLE &&
-      self->p_canceltype == PTHREAD_CANCEL_ASYNCHRONOUS)
+  if (oldstate != NULL) *oldstate = THREAD_GETMEM(self, p_cancelstate);
+  THREAD_SETMEM(self, p_cancelstate, state);
+  if (THREAD_GETMEM(self, p_canceled) &&
+      THREAD_GETMEM(self, p_cancelstate) == PTHREAD_CANCEL_ENABLE &&
+      THREAD_GETMEM(self, p_canceltype) == PTHREAD_CANCEL_ASYNCHRONOUS)
     pthread_exit(PTHREAD_CANCELED);
   return 0;
 }
@@ -39,11 +39,11 @@ int pthread_setcanceltype(int type, int * oldtype)
   pthread_descr self = thread_self();
   if (type < PTHREAD_CANCEL_DEFERRED || type > PTHREAD_CANCEL_ASYNCHRONOUS)
     return EINVAL;
-  if (oldtype != NULL) *oldtype = self->p_canceltype;
-  self->p_canceltype = type;
-  if (self->p_canceled &&
-      self->p_cancelstate == PTHREAD_CANCEL_ENABLE &&
-      self->p_canceltype == PTHREAD_CANCEL_ASYNCHRONOUS)
+  if (oldtype != NULL) *oldtype = THREAD_GETMEM(self, p_canceltype);
+  THREAD_SETMEM(self, p_canceltype, type);
+  if (THREAD_GETMEM(self, p_canceled) &&
+      THREAD_GETMEM(self, p_cancelstate) == PTHREAD_CANCEL_ENABLE &&
+      THREAD_GETMEM(self, p_canceltype) == PTHREAD_CANCEL_ASYNCHRONOUS)
     pthread_exit(PTHREAD_CANCELED);
   return 0;
 }
@@ -53,22 +53,23 @@ int pthread_cancel(pthread_t thread)
   pthread_handle handle = thread_handle(thread);
   int pid;
 
-  acquire(&handle->h_spinlock);
+  __pthread_lock(&handle->h_lock, NULL);
   if (invalid_handle(handle, thread)) {
-    release(&handle->h_spinlock);
+    __pthread_unlock(&handle->h_lock);
     return ESRCH;
   }
   handle->h_descr->p_canceled = 1;
   pid = handle->h_descr->p_pid;
-  release(&handle->h_spinlock);
-  kill(pid, PTHREAD_SIG_CANCEL);
+  __pthread_unlock(&handle->h_lock);
+   kill(pid, __pthread_sig_cancel);
   return 0;
 }
 
 void pthread_testcancel(void)
 {
   pthread_descr self = thread_self();
-  if (self->p_canceled && self->p_cancelstate == PTHREAD_CANCEL_ENABLE)
+  if (THREAD_GETMEM(self, p_canceled)
+      && THREAD_GETMEM(self, p_cancelstate) == PTHREAD_CANCEL_ENABLE)
     pthread_exit(PTHREAD_CANCELED);
 }
 
@@ -76,42 +77,42 @@ void _pthread_cleanup_push(struct _pthread_cleanup_buffer * buffer,
 			   void (*routine)(void *), void * arg)
 {
   pthread_descr self = thread_self();
-  buffer->routine = routine;
-  buffer->arg = arg;
-  buffer->prev = self->p_cleanup;
-  self->p_cleanup = buffer;
+  buffer->__routine = routine;
+  buffer->__arg = arg;
+  buffer->__prev = THREAD_GETMEM(self, p_cleanup);
+  THREAD_SETMEM(self, p_cleanup, buffer);
 }
 
 void _pthread_cleanup_pop(struct _pthread_cleanup_buffer * buffer,
 			  int execute)
 {
   pthread_descr self = thread_self();
-  if (execute) buffer->routine(buffer->arg);
-  self->p_cleanup = buffer->prev;
+  if (execute) buffer->__routine(buffer->__arg);
+  THREAD_SETMEM(self, p_cleanup, buffer->__prev);
 }
 
 void _pthread_cleanup_push_defer(struct _pthread_cleanup_buffer * buffer,
 				 void (*routine)(void *), void * arg)
 {
   pthread_descr self = thread_self();
-  buffer->routine = routine;
-  buffer->arg = arg;
-  buffer->canceltype = self->p_canceltype;
-  buffer->prev = self->p_cleanup;
-  self->p_canceltype = PTHREAD_CANCEL_DEFERRED;
-  self->p_cleanup = buffer;
+  buffer->__routine = routine;
+  buffer->__arg = arg;
+  buffer->__canceltype = THREAD_GETMEM(self, p_canceltype);
+  buffer->__prev = THREAD_GETMEM(self, p_cleanup);
+  THREAD_SETMEM(self, p_canceltype, PTHREAD_CANCEL_DEFERRED);
+  THREAD_SETMEM(self, p_cleanup, buffer);
 }
 
 void _pthread_cleanup_pop_restore(struct _pthread_cleanup_buffer * buffer,
 				  int execute)
 {
   pthread_descr self = thread_self();
-  if (execute) buffer->routine(buffer->arg);
-  self->p_cleanup = buffer->prev;
-  self->p_canceltype = buffer->canceltype;
-  if (self->p_canceled &&
-      self->p_cancelstate == PTHREAD_CANCEL_ENABLE &&
-      self->p_canceltype == PTHREAD_CANCEL_ASYNCHRONOUS)
+  if (execute) buffer->__routine(buffer->__arg);
+  THREAD_SETMEM(self, p_cleanup, buffer->__prev);
+  THREAD_SETMEM(self, p_canceltype, buffer->__canceltype);
+  if (THREAD_GETMEM(self, p_canceled) &&
+      THREAD_GETMEM(self, p_cancelstate) == PTHREAD_CANCEL_ENABLE &&
+      THREAD_GETMEM(self, p_canceltype) == PTHREAD_CANCEL_ASYNCHRONOUS)
     pthread_exit(PTHREAD_CANCELED);
 }
 
@@ -119,7 +120,8 @@ void __pthread_perform_cleanup(void)
 {
   pthread_descr self = thread_self();
   struct _pthread_cleanup_buffer * c;
-  for (c = self->p_cleanup; c != NULL; c = c->prev) c->routine(c->arg);
+  for (c = THREAD_GETMEM(self, p_cleanup); c != NULL; c = c->__prev)
+    c->__routine(c->__arg);
 }
 
 #ifndef PIC
