@@ -1,6 +1,6 @@
 /* Machine-dependent pthreads configuration and inline functions.
    Alpha version.
-   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Richard Henderson <rth@tamu.edu>.
 
@@ -19,13 +19,18 @@
    write to the Free Software Foundation, Inc.,  59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
+#ifndef PT_EI
+# define PT_EI extern inline
+#endif
+
 #include <asm/pal.h>
 
 
 /* Spinlock implementation; required.  */
-extern inline long testandset(int *spinlock)
+PT_EI long int
+testandset (int *spinlock)
 {
-  long ret, temp;
+  long int ret, temp;
 
   __asm__ __volatile__(
 	"/* Inline spinlock test & set */\n"
@@ -62,27 +67,28 @@ register char *stack_pointer __asm__("$30");
 
 
 /* Return the thread descriptor for the current thread.  */
-#define THREAD_SELF						\
-{								\
-  register pthread_handle __self __asm__("$0");			\
-  __asm__ ("call_pal %1" : "=r"(__self) : "i"(PAL_rduniq));	\
-  return __self;						\
-}
+#define THREAD_SELF \
+({									      \
+  register pthread_descr __self __asm__("$0");				      \
+  __asm__ ("call_pal %1" : "=r"(__self) : "i"(PAL_rduniq) : "$0");	      \
+  __self;								      \
+})
 
 /* Initialize the thread-unique value.  */
-#define INIT_THREAD_SELF(descr)						 \
-{									 \
-  register pthread_handle __self __asm__("$16") = (descr);		 \
-  __asm__ __volatile__ ("call_pal %1" : : "r"(__self), "i"(PAL_wruniq)); \
+#define INIT_THREAD_SELF(descr, nr) \
+{									      \
+  register pthread_descr __self __asm__("$16") = (descr);		      \
+  __asm__ __volatile__ ("call_pal %1" : : "r"(__self), "i"(PAL_wruniq));      \
 }
 
 
 /* Compare-and-swap for semaphores. */
 
 #define HAS_COMPARE_AND_SWAP
-extern inline int __compare_and_swap(long * p, long oldval, long newval)
+PT_EI int
+__compare_and_swap (long int *p, long int oldval, long int newval)
 {
-  long ret;
+  long int ret;
 
   __asm__ __volatile__ (
 	"/* Inline compare & swap */\n"
