@@ -1,6 +1,6 @@
 /* Machine-dependent pthreads configuration and inline functions.
    i386 version.
-   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Richard Henderson <rth@tamu.edu>.
 
@@ -19,16 +19,21 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
+#ifndef PT_EI
+# define PT_EI extern inline
+#endif
 
 /* Spinlock implementation; required.  */
-extern inline int
+PT_EI int
 testandset (int *spinlock)
 {
   int ret;
 
-  __asm__ __volatile__("xchgl %0, %1"
-	: "=r"(ret), "=m"(*spinlock)
-	: "0"(1), "m"(*spinlock));
+  __asm__ __volatile__(
+       "xchgl %0, %1"
+       : "=r"(ret), "=m"(*spinlock)
+       : "0"(1), "m"(*spinlock)
+       : "memory");
 
   return ret;
 }
@@ -47,7 +52,7 @@ register char * stack_pointer __asm__ ("%esp");
 #define HAS_COMPARE_AND_SWAP
 #define TEST_FOR_COMPARE_AND_SWAP
 
-extern inline int
+PT_EI int
 __compare_and_swap (long int *p, long int oldval, long int newval)
 {
   char ret;
@@ -55,12 +60,13 @@ __compare_and_swap (long int *p, long int oldval, long int newval)
 
   __asm__ __volatile__ ("lock; cmpxchgl %3, %1; sete %0"
 			: "=q" (ret), "=m" (*p), "=a" (readval)
-			: "r" (newval), "m" (*p), "a" (oldval));
+			: "r" (newval), "m" (*p), "a" (oldval)
+			: "memory");
   return ret;
 }
 
 
-extern inline int
+PT_EI int
 get_eflags (void)
 {
   int res;
@@ -69,14 +75,14 @@ get_eflags (void)
 }
 
 
-extern inline void
+PT_EI void
 set_eflags (int newflags)
 {
   __asm__ __volatile__ ("pushl %0; popfl" : : "r" (newflags) : "cc");
 }
 
 
-extern inline int
+PT_EI int
 compare_and_swap_is_available (void)
 {
   int oldflags = get_eflags ();
