@@ -13,136 +13,122 @@
 /* GNU Library General Public License for more details.                 */
 
 #ifndef _PTHREAD_H
-
 #define _PTHREAD_H	1
+
 #include <features.h>
 
-#include <errno.h>
-#include <limits.h>
 #include <sched.h>
-#include <unistd.h>
+#include <time.h>
 
 #define __need_sigset_t
 #include <signal.h>
-#define __need_timespec
-#include <time.h>
-
-/* Linux has no ENOTSUP error code.  */
-#ifndef ENOTSUP
-#define ENOTSUP EOPNOTSUPP
-#endif
+#include <bits/pthreadtypes.h>
 
 
 __BEGIN_DECLS
 
-/*** Types ***/
-
-/* Thread identifiers */
-typedef unsigned long int pthread_t;
-
-/* Thread descriptors */
-typedef struct _pthread_descr_struct *_pthread_descr;
-
-/* Waiting queues (not abstract because mutexes and conditions aren't).  */
-struct _pthread_queue
-{
-  _pthread_descr head;		/* First element, or NULL if queue empty.  */
-  _pthread_descr tail;		/* Last element, or NULL if queue empty.  */
-};
-
-/* Mutexes (not abstract because of PTHREAD_MUTEX_INITIALIZER).  */
-typedef struct
-{
-  int m_spinlock;		/* Spin lock to guarantee mutual exclusion.  */
-  int m_count;			/* 0 if free, > 0 if taken.  */
-  _pthread_descr m_owner;	/* Owner of mutex (for recursive mutexes) */
-  int m_kind;			/* Kind of mutex */
-  struct _pthread_queue m_waiting; /* Threads waiting on this mutex.  */
-} pthread_mutex_t;
+/* Initializers.  */
 
 #define PTHREAD_MUTEX_INITIALIZER \
   {0, 0, 0, PTHREAD_MUTEX_FAST_NP, {0, 0}}
-#define PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP \
+#ifdef __USE_GNU
+# define PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP \
   {0, 0, 0, PTHREAD_MUTEX_RECURSIVE_NP, {0, 0}}
+#endif
 
-/* Conditions (not abstract because of PTHREAD_COND_INITIALIZER */
-typedef struct
-{
-  int c_spinlock;                  /* Spin lock to protect the queue.  */
-  struct _pthread_queue c_waiting; /* Threads waiting on this condition.  */
-} pthread_cond_t;
+#define PTHREAD_COND_INITIALIZER {{0, 0}, 0}
 
-#define PTHREAD_COND_INITIALIZER {0, {0, 0}}
+#ifdef __USE_UNIX98
+# define PTHREAD_RWLOCK_INITIALIZER \
+  { {0, 0}, 0, NULL, NULL, NULL,					      \
+    PTHREAD_RWLOCK_DEFAULT_NP, PTHREAD_PROCESS_PRIVATE }
+#endif
 
-/* Attributes */
+/* Values for attributes.  */
 
 enum
 {
   PTHREAD_CREATE_JOINABLE,
+#define PTHREAD_CREATE_JOINABLE	PTHREAD_CREATE_JOINABLE
   PTHREAD_CREATE_DETACHED
+#define PTHREAD_CREATE_DETACHED	PTHREAD_CREATE_DETACHED
 };
 
 enum
 {
   PTHREAD_INHERIT_SCHED,
+#define PTHREAD_INHERIT_SCHED	PTHREAD_INHERIT_SCHED
   PTHREAD_EXPLICIT_SCHED
+#define PTHREAD_EXPLICIT_SCHED	PTHREAD_EXPLICIT_SCHED
 };
 
 enum
 {
   PTHREAD_SCOPE_SYSTEM,
+#define PTHREAD_SCOPE_SYSTEM	PTHREAD_SCOPE_SYSTEM
   PTHREAD_SCOPE_PROCESS
+#define PTHREAD_SCOPE_PROCESS	PTHREAD_SCOPE_PROCESS
 };
 
-typedef struct
-{
-  int detachstate;
-  int schedpolicy;
-  struct sched_param schedparam;
-  int inheritsched;
-  int scope;
-} pthread_attr_t;
-
 enum
 {
   PTHREAD_MUTEX_FAST_NP,
-  PTHREAD_MUTEX_RECURSIVE_NP
+  PTHREAD_MUTEX_RECURSIVE_NP,
+  PTHREAD_MUTEX_ERRORCHECK_NP
+#ifdef __USE_UNIX98
+  ,
+  PTHREAD_MUTEX_NORMAL = PTHREAD_MUTEX_FAST_NP,
+  PTHREAD_MUTEX_RECURSIVE = PTHREAD_MUTEX_RECURSIVE_NP,
+  PTHREAD_MUTEX_ERRORCHECK = PTHREAD_MUTEX_ERRORCHECK_NP,
+  PTHREAD_MUTEX_DEFAULT = PTHREAD_MUTEX_NORMAL
+#endif
 };
 
-typedef struct
+enum
 {
-  int mutexkind;
-} pthread_mutexattr_t;
+  PTHREAD_PROCESS_PRIVATE,
+#define PTHREAD_PROCESS_PRIVATE	PTHREAD_PROCESS_PRIVATE
+  PTHREAD_PROCESS_SHARED
+#define PTHREAD_PROCESS_SHARED	PTHREAD_PROCESS_SHARED
+};
 
-typedef struct
+#ifdef __USE_UNIX98
+enum
 {
-  int dummy;
-} pthread_condattr_t;
-
-
-/* Keys for thread-specific data */
-
-typedef unsigned int pthread_key_t;
-
-/* Once-only execution */
-
-typedef int pthread_once_t;
+  PTHREAD_RWLOCK_PREFER_READER_NP,
+  PTHREAD_RWLOCK_PREFER_WRITER_NP,
+  PTHREAD_RWLOCK_DEFAULT_NP = PTHREAD_RWLOCK_PREFER_WRITER_NP
+};
+#endif	/* Unix98 */
 
 #define PTHREAD_ONCE_INIT 0
 
 /* Cleanup buffers */
 
-struct _pthread_cleanup_buffer {
-  void (*routine) __P ((void *));	/* Function to call.  */
-  void *arg;				/* Its argument.  */
-  int canceltype;			/* Saved cancellation type. */
-  struct _pthread_cleanup_buffer *prev;	/* Chaining of cleanup functions.  */
+struct _pthread_cleanup_buffer
+{
+  void (*__routine) __PMT ((void *));	  /* Function to call.  */
+  void *__arg;				  /* Its argument.  */
+  int __canceltype;			  /* Saved cancellation type. */
+  struct _pthread_cleanup_buffer *__prev; /* Chaining of cleanup functions.  */
 };
 
 /* Cancellation */
 
-enum { PTHREAD_CANCEL_ENABLE, PTHREAD_CANCEL_DISABLE };
-enum { PTHREAD_CANCEL_DEFERRED, PTHREAD_CANCEL_ASYNCHRONOUS };
+enum
+{
+  PTHREAD_CANCEL_ENABLE,
+#define PTHREAD_CANCEL_ENABLE	PTHREAD_CANCEL_ENABLE
+  PTHREAD_CANCEL_DISABLE
+#define PTHREAD_CANCEL_DISABLE	PTHREAD_CANCEL_DISABLE
+};
+enum
+{
+  PTHREAD_CANCEL_DEFERRED,
+#define PTHREAD_CANCEL_DEFERRED	PTHREAD_CANCEL_DEFERRED
+  PTHREAD_CANCEL_ASYNCHRONOUS
+#define PTHREAD_CANCEL_ASYNCHRONOUS	PTHREAD_CANCEL_ASYNCHRONOUS
+};
 #define PTHREAD_CANCELED ((void *) -1)
 
 
@@ -163,7 +149,7 @@ extern pthread_t pthread_self __P ((void));
 extern int pthread_equal __P ((pthread_t __thread1, pthread_t __thread2));
 
 /* Terminate calling thread.  */
-extern void pthread_exit __P ((void *__retval)) __attribute__ ((noreturn));
+extern void pthread_exit __P ((void *__retval)) __attribute__ ((__noreturn__));
 
 /* Make calling thread wait for termination of the thread TH.  The
    exit status of the thread is stored in *THREAD_RETURN, if THREAD_RETURN
@@ -180,7 +166,8 @@ extern int pthread_detach __P ((pthread_t __th));
 /* Functions for handling attributes.  */
 
 /* Initialize thread attribute *ATTR with default attributes
-   (detachstate is PTHREAD_JOINABLE, scheduling policy is SCHED_OTHER).  */
+   (detachstate is PTHREAD_JOINABLE, scheduling policy is SCHED_OTHER,
+    no user-provided stack).  */
 extern int pthread_attr_init __P ((pthread_attr_t *__attr));
 
 /* Destroy thread attribute *ATTR.  */
@@ -225,6 +212,37 @@ extern int pthread_attr_setscope __P ((pthread_attr_t *__attr, int __scope));
 extern int pthread_attr_getscope __P ((__const pthread_attr_t *__attr,
 				       int *__scope));
 
+#ifdef __USE_UNIX98
+/* Set the size of the guard area at the bottom of the thread.  */
+extern int pthread_attr_setguardsize __P ((pthread_attr_t *__attr,
+					   size_t __guardsize));
+
+/* Get the size of the guard area at the bottom of the thread.  */
+extern int pthread_attr_getguardsize __P ((__const pthread_attr_t *__attr,
+					   size_t *__guardsize));
+#endif
+
+/* Set the starting address of the stack of the thread to be created.
+   Depending on whether the stack grows up or doen the value must either
+   be higher or lower than all the address in the memory block.  The
+   minimal size of the block must be PTHREAD_STACK_SIZE.  */
+extern int pthread_attr_setstackaddr __P ((pthread_attr_t *__attr,
+					   void *__stackaddr));
+
+/* Return the previously set address for the stack.  */
+extern int pthread_attr_getstackaddr __P ((__const pthread_attr_t *__attr,
+					   void **__stackaddr));
+
+/* Add information about the minimum stack size needed for the thread
+   to be started.  This size must never be less than PTHREAD_STACK_SIZE
+   and must also not exceed the system limits.  */
+extern int pthread_attr_setstacksize __P ((pthread_attr_t *__attr,
+					   size_t __stacksize));
+
+/* Return the currently used minimal stack size.  */
+extern int pthread_attr_getstacksize __P ((__const pthread_attr_t *__attr,
+					   size_t *__stacksize));
+
 /* Functions for scheduling control. */
 
 /* Set the scheduling parameters for TARGET_THREAD according to POLICY
@@ -237,6 +255,13 @@ extern int pthread_getschedparam __P ((pthread_t __target_thread,
 				       int *__policy,
 				       struct sched_param *__param));
 
+#ifdef __USE_UNIX98
+/* Determine  level of concurrency.  */
+extern int pthread_getconcurrency __P ((void));
+
+/* Set new concurrency level to LEVEL.  */
+extern int pthread_setconcurrency __P ((int __level));
+#endif
 
 /* Functions for mutex handling. */
 
@@ -275,15 +300,19 @@ extern int pthread_mutexattr_init __P ((pthread_mutexattr_t *__attr));
 extern int __pthread_mutexattr_destroy __P ((pthread_mutexattr_t *__attr));
 extern int pthread_mutexattr_destroy __P ((pthread_mutexattr_t *__attr));
 
-/* Set the mutex kind attribute in *ATTR to KIND (either PTHREAD_MUTEX_FAST_NP
-   or PTHREAD_MUTEX_RECURSIVE_NP). */
-extern int __pthread_mutexattr_setkind_np __P ((pthread_mutexattr_t *__attr,
-						int __kind));
-extern int pthread_mutexattr_setkind_np __P ((pthread_mutexattr_t *__attr,
-					      int __kind));
+#ifdef __USE_UNIX98
+/* Set the mutex kind attribute in *ATTR to KIND (either PTHREAD_MUTEX_NORMAL,
+   PTHREAD_MUTEX_RECURSIVE, PTHREAD_MUTEX_ERRORCHECK, or
+   PTHREAD_MUTEX_DEFAULT).  */
+extern int __pthread_mutexattr_settype __P ((pthread_mutexattr_t *__attr,
+					     int __kind));
+extern int pthread_mutexattr_settype __P ((pthread_mutexattr_t *__attr,
+					   int __kind));
+
 /* Return in *KIND the mutex kind attribute in *ATTR. */
-extern int pthread_mutexattr_getkind_np __P ((__const pthread_mutexattr_t *__attr,
-                                              int *__kind));
+extern int pthread_mutexattr_gettype __P ((__const pthread_mutexattr_t *__attr,
+					   int *__kind));
+#endif
 
 
 /* Functions for handling conditional variables.  */
@@ -324,6 +353,61 @@ extern int pthread_condattr_init __P ((pthread_condattr_t *__attr));
 extern int pthread_condattr_destroy __P ((pthread_condattr_t *__attr));
 
 
+#ifdef __USE_UNIX98
+/* Functions for handling read-write locks.  */
+
+/* Initialize read-write lock RWLOCK using attributes ATTR, or use
+   the default values if later is NULL.  */
+extern int pthread_rwlock_init __P ((pthread_rwlock_t *__rwlock,
+				     __const pthread_rwlockattr_t *__attr));
+
+/* Destroy read-write lock RWLOCK.  */
+extern int pthread_rwlock_destroy __P ((pthread_rwlock_t *__rwlock));
+
+/* Acquire read lock for RWLOCK.  */
+extern int pthread_rwlock_rdlock __P ((pthread_rwlock_t *__rwlock));
+
+/* Try to acquire read lock for RWLOCK.  */
+extern int pthread_rwlock_tryrdlock __P ((pthread_rwlock_t *__rwlock));
+
+/* Acquire write lock for RWLOCK.  */
+extern int pthread_rwlock_wrlock __P ((pthread_rwlock_t *__rwlock));
+
+/* Try to acquire writelock for RWLOCK.  */
+extern int pthread_rwlock_trywrlock __P ((pthread_rwlock_t *__rwlock));
+
+/* Unlock RWLOCK.  */
+extern int pthread_rwlock_unlock __P ((pthread_rwlock_t *__rwlock));
+
+
+/* Functions for handling read-write lock attributes.  */
+
+/* Initialize attribute object ATTR with default values.  */
+extern int pthread_rwlockattr_init __P ((pthread_rwlockattr_t *__attr));
+
+/* Destroy attribute object ATTR.  */
+extern int pthread_rwlockattr_destroy __P ((pthread_rwlockattr_t *__attr));
+
+/* Return current setting of process-shared attribute of ATTR in PSHARED.  */
+extern int pthread_rwlockattr_getpshared __P ((__const
+					       pthread_rwlockattr_t *__attr,
+					       int *__pshared));
+
+/* Set process-shared attribute of ATTR to PSHARED.  */
+extern int pthread_rwlockattr_setpshared __P ((pthread_rwlockattr_t *__attr,
+					       int __pshared));
+
+/* Return current setting of reader/writer preference.  */
+extern int pthread_rwlockattr_getkind_np __P ((__const
+					       pthread_rwlockattr_t *__attr,
+					       int *__pref));
+
+/* Set reader/write preference.  */
+extern int pthread_rwlockattr_setkind_np __P ((pthread_rwlockattr_t *__attr,
+					       int __pref));
+#endif
+
+
 /* Functions for handling thread-specific data */
 
 /* Create a key value identifying a location in the thread-specific data
@@ -357,6 +441,8 @@ extern void *pthread_getspecific __P ((pthread_key_t __key));
    only once, even if pthread_once is executed several times with the
    same ONCE_CONTROL argument. ONCE_CONTROL must point to a static or
    extern variable initialized to PTHREAD_ONCE_INIT. */
+extern int __pthread_once __P ((pthread_once_t *__once_control,
+				void (*__init_routine) (void)));
 extern int pthread_once __P ((pthread_once_t *__once_control,
 			      void (*__init_routine) (void)));
 
@@ -369,7 +455,6 @@ extern int pthread_setcancelstate __P ((int __state, int *__oldstate));
 
 /* Set cancellation state of current thread to TYPE, returning the old
    type in *OLDTYPE if OLDTYPE is not NULL.  */
-extern int __pthread_setcanceltype __P ((int __type, int *__oldtype));
 extern int pthread_setcanceltype __P ((int __type, int *__oldtype));
 
 /* Cancel THREAD immediately or at the next possibility.  */
@@ -408,7 +493,8 @@ extern void _pthread_cleanup_pop __P ((struct _pthread_cleanup_buffer *__buffer,
 /* Install a cleanup handler as pthread_cleanup_push does, but also
    saves the current cancellation type and set it to deferred cancellation. */
 
-#define pthread_cleanup_push_defer_np(routine,arg)			      \
+#ifdef __USE_GNU
+# define pthread_cleanup_push_defer_np(routine,arg)			      \
   { struct _pthread_cleanup_buffer _buffer;				      \
     _pthread_cleanup_push_defer (&_buffer, (routine), (arg));
 
@@ -420,23 +506,15 @@ extern void _pthread_cleanup_push_defer __P ((struct _pthread_cleanup_buffer *__
    restores the cancellation type that was in effect when the matching
    pthread_cleanup_push_defer was called. */
 
-#define pthread_cleanup_pop_restore_np(execute)				      \
+# define pthread_cleanup_pop_restore_np(execute)			      \
   _pthread_cleanup_pop_restore (&_buffer, (execute)); }
 
 extern void _pthread_cleanup_pop_restore __P ((struct _pthread_cleanup_buffer *__buffer,
 					       int __execute));
+#endif
 
 /* Functions for handling signals. */
-
-/* Modify the signal mask for the calling thread.  The arguments have
-   the same meaning as for sigprocmask(2). */
-
-extern int pthread_sigmask __P ((int __how, __const sigset_t *__newmask,
-				 sigset_t *__oldmask));
-
-/* Send signal SIGNO to the given thread. */
-
-extern int pthread_kill __P ((pthread_t __thread, int __signo));
+#include <bits/sigthread.h>
 
 
 /* Functions for handling process creation and process execution. */
@@ -452,6 +530,9 @@ extern int pthread_kill __P ((pthread_t __thread, int __signo));
    first called before FORK), and the PARENT and CHILD handlers are called
    in FIFO (first added, first called). */
 
+extern int __pthread_atfork __P ((void (*__prepare) (void),
+				  void (*__parent) (void),
+				  void (*__child) (void)));
 extern int pthread_atfork __P ((void (*__prepare) (void),
 				void (*__parent) (void),
 				void (*__child) (void)));
@@ -459,7 +540,6 @@ extern int pthread_atfork __P ((void (*__prepare) (void),
 /* Terminate all threads in the program except the calling process.
    Should be called just before invoking one of the exec*() functions. */
 
-extern void __pthread_kill_other_threads_np __P ((void));
 extern void pthread_kill_other_threads_np __P ((void));
 
 
