@@ -1,5 +1,5 @@
 /* The weak pthread functions for Linux.
-   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -21,15 +21,20 @@
 #include <limits.h>
 #include <stdlib.h>
 
-/* This is from <pthread.h>.  */
-typedef unsigned int pthread_key_t;
-
 extern int __pthread_return_0 __P ((void));
 extern int __pthread_return_1 __P ((void));
 extern void __pthread_return_void __P ((void));
 
 /* Those are pthread functions which return 0 if successful. */
+#if defined HAVE_ELF && defined PIC && defined DO_VERSIONING
+weak_alias (__pthread_return_0, __libc_pthread_attr_init_2_0)
+symbol_version (__libc_pthread_attr_init_2_0, pthread_attr_init, GLIBC_2.0);
+weak_alias (__pthread_return_0, __libc_pthread_attr_init_2_1)
+default_symbol_version (__libc_pthread_attr_init_2_1, pthread_attr_init,
+			GLIBC_2.1);
+#else
 weak_alias (__pthread_return_0, pthread_attr_init)
+#endif
 weak_alias (__pthread_return_0, pthread_attr_destroy)
 weak_alias (__pthread_return_0, pthread_attr_setdetachstate)
 weak_alias (__pthread_return_0, pthread_attr_getdetachstate)
@@ -41,24 +46,46 @@ weak_alias (__pthread_return_0, pthread_attr_setinheritsched)
 weak_alias (__pthread_return_0, pthread_attr_getinheritsched)
 weak_alias (__pthread_return_0, pthread_attr_setscope)
 weak_alias (__pthread_return_0, pthread_attr_getscope)
+weak_alias (__pthread_return_0, pthread_attr_setstackaddr)
+weak_alias (__pthread_return_0, pthread_attr_getstackaddr)
+weak_alias (__pthread_return_0, pthread_attr_setstacksize)
+weak_alias (__pthread_return_0, pthread_attr_getstacksize)
 weak_alias (__pthread_return_0, pthread_mutex_init)
 weak_alias (__pthread_return_0, pthread_mutex_destroy)
 weak_alias (__pthread_return_0, pthread_mutex_lock)
+weak_alias (__pthread_return_0, pthread_mutex_trylock)
 weak_alias (__pthread_return_0, pthread_mutex_unlock)
-weak_alias (__pthread_return_0, pthread_mutexattr_setkind_np)
-weak_alias (__pthread_return_0, pthread_mutexattr_getkind_np)
+weak_alias (__pthread_return_0, pthread_mutexattr_init)
+weak_alias (__pthread_return_0, pthread_mutexattr_destroy)
+weak_alias (__pthread_return_0, pthread_mutexattr_settype)
+weak_alias (__pthread_return_0, pthread_mutexattr_gettype)
 weak_alias (__pthread_return_0, pthread_condattr_init)
 weak_alias (__pthread_return_0, pthread_condattr_destroy)
 weak_alias (__pthread_return_0, pthread_setschedparam)
 weak_alias (__pthread_return_0, pthread_getschedparam)
+weak_alias (__pthread_return_0, pthread_getcancelstate)
 weak_alias (__pthread_return_0, pthread_setcancelstate)
 weak_alias (__pthread_return_0, pthread_setcanceltype)
+weak_alias (__pthread_return_0, pthread_setconcurrency)
+weak_alias (__pthread_return_0, pthread_getconcurrency)
 weak_alias (__pthread_return_0, pthread_self)
 weak_alias (__pthread_return_0, pthread_cond_init)
 weak_alias (__pthread_return_0, pthread_cond_destroy)
 weak_alias (__pthread_return_0, pthread_cond_wait)
+weak_alias (__pthread_return_0, pthread_cond_timedwait)
 weak_alias (__pthread_return_0, pthread_cond_signal)
 weak_alias (__pthread_return_0, pthread_cond_broadcast)
+weak_alias (__pthread_return_0, pthread_rwlock_init)
+weak_alias (__pthread_return_0, pthread_rwlock_destroy)
+weak_alias (__pthread_return_0, pthread_rwlock_rdlock)
+weak_alias (__pthread_return_0, pthread_rwlock_wrlock)
+weak_alias (__pthread_return_0, pthread_rwlock_tryrdlock)
+weak_alias (__pthread_return_0, pthread_rwlock_trywrlock)
+weak_alias (__pthread_return_0, pthread_rwlock_unlock)
+weak_alias (__pthread_return_0, pthread_rwlockattr_init)
+weak_alias (__pthread_return_0, pthread_rwlockattr_destroy)
+weak_alias (__pthread_return_0, pthread_rwlockattr_setpshared)
+weak_alias (__pthread_return_0, pthread_rwlockattr_getpshared)
 
 
 /* Those are pthread functions which return 1 if successful. */
@@ -88,74 +115,3 @@ void
 __pthread_return_void (void)
 {
 }
-
-
-/* The key and thread-specific data functions cannot simply be dummies
-   since data might be stored in any case.  */
-
-/* Number of supported keys.  */
-
-/* Table of keys. */
-struct pthread_key_struct
-  {
-    int occupied;
-    void (*destr) (void *);
-    const void *data;
-  };
-
-static struct pthread_key_struct keys[PTHREAD_KEYS_MAX] =
-  {
-    { 0, NULL, NULL },
-  };
-
-
-int
-__attribute__ ((weak))
-__pthread_key_create (pthread_key_t *key, void (*destructor) (void *))
-{
-  unsigned int cnt;
-
-  for (cnt = 0; cnt < PTHREAD_KEYS_MAX; ++cnt)
-    if (! keys[cnt].occupied)
-      {
-	keys[cnt].occupied = 1;
-	keys[cnt].destr = destructor;
-	*key = cnt;
-	return 0;
-      }
-  return EAGAIN;
-}
-
-
-int
-__attribute__ ((weak))
-__pthread_key_delete (pthread_key_t key)
-{
-  if (key >= PTHREAD_KEYS_MAX || ! keys[key].occupied)
-    return EINVAL;
-  keys[key].occupied = 0;
-  keys[key].destr = NULL;
-  keys[key].data = NULL;
-  return 0;
-}
-
-
-int
-__attribute__ ((weak))
-__pthread_setspecific (pthread_key_t key, const void *ptr)
-{
-  if (key >= PTHREAD_KEYS_MAX || ! keys[key].occupied)
-    return EINVAL;
-  keys[key].data = ptr;
-  return 0;
-}
-
-
-void *
-__attribute__ ((weak))
-__pthread_getspecific (pthread_key_t key)
-{
-  if (key >= PTHREAD_KEYS_MAX || ! keys[key].occupied)
-    return NULL;
-  return (void *) keys[key].data;
-}
