@@ -1,5 +1,5 @@
-/* wrapsyscall - wrapper arpund system calls to provide cancelation points.
-   Copyright (C) 1996 Free Software Foundation, Inc.
+/* Wrapper arpund system calls to provide cancelation points.
+   Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Ulrich Drepper <drepper@cygnus.com>, 1996.
 
@@ -19,12 +19,16 @@
    Boston, MA 02111-1307, USA.  */
 
 #include <fcntl.h>
+#include <sys/mman.h>
 #include <pthread.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <stddef.h>
+#include <stdlib.h>
+#include <termios.h>
 #include <sys/resource.h>
 #include <sys/wait.h>
+#include <sys/socket.h>
 
 
 #ifndef PIC
@@ -86,8 +90,8 @@ strong_alias (lseek, __lseek)
 
 
 /* msync(2).  */
-CANCELABLE_SYSCALL (int, msync, (const void *start, size_t length, int flags),
-		    (start, length, flags))
+CANCELABLE_SYSCALL (int, msync, (__ptr_t addr, size_t length, int flags),
+		    (addr, length, flags))
 
 
 /* nanosleep(2).  */
@@ -135,3 +139,48 @@ CANCELABLE_SYSCALL (__pid_t, waitpid, (__pid_t pid, int *stat_loc,
 CANCELABLE_SYSCALL (ssize_t, write, (int fd, const void *buf, size_t n),
 		    (fd, buf, n))
 strong_alias (write, __write)
+
+
+/* The following system calls are thread cancellation points specified
+   in XNS.  */
+
+/* accept(2).  */
+CANCELABLE_SYSCALL (int, accept, (int fd, __SOCKADDR_ARG addr,
+				  socklen_t *addr_len),
+		    (fd, addr, addr_len))
+
+/* connect(2).  */
+CANCELABLE_SYSCALL (int, connect, (int fd, __CONST_SOCKADDR_ARG addr,
+				     socklen_t len),
+		    (fd, addr, len))
+strong_alias (connect, __connect)
+
+/* recv(2).  */
+CANCELABLE_SYSCALL (int, recv, (int fd, __ptr_t buf, size_t n, int flags),
+		    (fd, buf, n, flags))
+
+/* recvfrom(2).  */
+CANCELABLE_SYSCALL (int, recvfrom, (int fd, __ptr_t buf, size_t n, int flags,
+				    __SOCKADDR_ARG addr, socklen_t *addr_len),
+		    (fd, buf, n, flags, addr, addr_len))
+
+/* recvmsg(2).  */
+CANCELABLE_SYSCALL (int, recvmsg, (int fd, struct msghdr *message, int flags),
+		    (fd, message, flags))
+
+/* send(2).  */
+CANCELABLE_SYSCALL (int, send, (int fd, const __ptr_t buf, size_t n,
+				int flags),
+		    (fd, buf, n, flags))
+strong_alias (send, __send)
+
+/* sendmsg(2).  */
+CANCELABLE_SYSCALL (int, sendmsg, (int fd, const struct msghdr *message,
+				   int flags),
+		    (fd, message, flags))
+
+/* sendto(2).  */
+CANCELABLE_SYSCALL (int, sendto, (int fd, const __ptr_t buf, size_t n,
+				  int flags, __CONST_SOCKADDR_ARG addr,
+				  socklen_t addr_len),
+		    (fd, buf, n, flags, addr, addr_len))
